from azure.cognitiveservices.vision.face import FaceClient
from msrest.authentication import CognitiveServicesCredentials
import pandas as pd
from PIL import Image
import matplotlib.pyplot as plt
import engine
import os
import time
import datetime
import streamlit as st
import take_photo
from streamlit import caching



def print_current_groups(app_mode):
    st.header(app_mode)
    try:
        df = pd.read_csv('group_id.csv')
        st.dataframe(df)
    except:
        st.error('The group list is Empty')
    # group_API = face_client.person_group.list()
    # for group in group_API:
    #     group
def create_a_group_id(app_mode):
    column_list = ['ID']
    st.header(app_mode)

    try:
        df = pd.read_csv('group_id.csv', index_col=False)
        st.subheader('Current Groups')
        st.dataframe(df)
    except:
        st.error('The group list is Empty')
        df = pd.DataFrame(columns=column_list)
    st.subheader("New Group Creation")
    group_id = st.text_input('Write the New Group')
    if group_id in df.values:
        st.warning('Group already registered')
        st.stop()

    create = st.button("Create")

    if not create or not group_id:
        st.warning('Click in Create when all parameter are set')
        st.stop()
    st.success(f'Creating new group {group_id}')
    try:
        face_client.person_group.create(group_id, 'CZU')
        progress_bar = st.progress(0)
        status_text = st.empty()
        for i in range(1, 101):
            status_text.text("%i%% Complete" % i)
            progress_bar.progress(i)
            time.sleep(0.05)

        progress_bar.empty()
        st.success('Group created!')
    except Exception as ex:
        st.warning(ex)
    finally:
        df = df.append(pd.DataFrame(list([group_id]), columns=column_list), ignore_index=True)

    df.to_csv("group_id.csv", index=False)
    # create = input("Would you like to create a new one(Y/N)?")
    # if create.lower() == 'y':
    #     group_id = input("Name: ")
    #     try:
    #         face_client.person_group.create(group_id, 'CZU')
    #         print('Group created!')
    #     except Exception as ex:
    #         print(ex.message)
    #     finally:
    #         df = df.append(pd.DataFrame(list([group_id]), columns=column_list), ignore_index=True)
    #
    #     df.to_csv("group_id.csv", index=False)
    # elif create.lower() == 'n':
    #     return
    # else:
    #     print('This is no valid option')
    #     return

def add_new_student(app_mode):

    st.header(app_mode)
    df = pd.read_csv('group_id.csv')
    groups = df['ID'].tolist()
    group_id = st.selectbox('Select the Group list', groups)
    column_list = ['ID']
    new = st.sidebar.button('New')
    caching.clear_cache()
    try:
        df = pd.read_csv('student_id.csv')
    except:
        df = pd.DataFrame(columns=column_list)

    student_code = st.text_input(f'Write the {group_id} Code')
    if student_code in df.values:
        st.warning(f'{group_id} already registered, try with a new code')
        st.stop()

    if len(student_code) < 5:
        st.stop()
    else:
        uploading_photos(student_code, group_id)


def uploading_photos(student_code, group_id):
    column_list = ['ID']
    st.text(f'Please attach 3 photos from the student {student_code} according to the example')
    st.image('Example/example.png', use_column_width=True)
    uploaded_file = st.file_uploader('Upload Photos here', accept_multiple_files=True)

    if not uploaded_file:
        st.stop()
    else:
        try:
            os.mkdir(f'photos/{student_code}')
        except:
            pass
        for file in uploaded_file:
            with open(os.path.join('photos', student_code, file.name), "wb") as f:
                f.write(file.getbuffer())
    create = st.button("Create New Member")

    if not create or not student_code or not uploaded_file:
        st.warning('Click in Create when all parameter are set')
        st.stop()
    st.success(f'Creating new member of {group_id}')
    if student_code in df.values:
        print('Student already registered')
    else:
        #new_student = face_client.person_group_person.create(group_id, student_code)
        # Get photo's from the photos folder
        folder = os.path.join('photos', student_code)
        student_pics = os.listdir(folder)
        i = 0
        fig = plt.figure(figsize=(8, 8))
        for pic in student_pics:
            # Add each photo to person in person group
            img_path = os.path.join(folder, pic)
            img_stream = open(img_path, "rb")
            #face_client.person_group_person.add_face_from_stream(group_id, new_student.person_id, img_stream)

            # Display each image
            img = Image.open(img_path)
            i += 1
            a = fig.add_subplot(1, len(student_pics), i)
            a.axis('off')
            imgplot = plt.imshow(img)
            print('Training Process.....')
        plt.show()
        #face_client.person_group.train(group_id)
        print('Trained!')
        df = df.append(pd.DataFrame(list([student_code]), columns=column_list), ignore_index=True)

    #df.to_csv("student_id.csv", index=False)
    print('-------------------------------------')


def detect_student_class(app_mode):
    st.header(app_mode)
    auto = st.sidebar.radio(
        "What mode would like take the attendance?",
        ('Auto', 'Scheduled', 'Manual'))
    run = st.sidebar.button('Run')

    if auto == 'Auto':
        minutes = st.sidebar.slider('Select the interval (in minutes) to take attendance: ', 2, 25, 2)
        stop = st.sidebar.button("Stop")
        st.write(f'You will take attendance each {minutes} minutes')

    elif auto == 'Manual':
        camera = st.sidebar.radio(
            "Would you like to : ",
            ('Use built-in Camera', 'Upload a Photo'))
        if camera == 'Upload a Photo':
            uploaded_file = st.sidebar.file_uploader('Upload a Photo here')
        stop = True #Run just one time

    elif auto == 'Scheduled':
        appointment = st.sidebar.slider("Schedule your Class Time:", value=(datetime.time(11, 30), datetime.time(12, 45)))
        st.write(f"You scheduled attendance app from {appointment[0]} to {appointment[1]}")
        minutes = st.sidebar.slider('Select the interval (in minutes) to take attendance: ', 2, 25, 2)
        stop = st.sidebar.button("Stop")
        st.write(f'You will take attendance each {minutes} minutes')
    if not run:
        st.warning('Click in Run when all parameter are set')
        st.stop()
    st.success('Running the app')
    while True:
        take_photo.capture_write()
        image = Image.open("Test/image.jpeg")
        st.image(image, caption='Cam Picture', use_column_width=True)
        group_id = "student"
        # Get the face IDs in a second image
        image_path = os.path.join('Test', 'image.jpeg')
        image_stream = open(image_path, "rb")
        image_faces = face_client.face.detect_with_stream(image=image_stream, detection_model='detection_03')
        image_face_ids = list(map(lambda face: face.face_id, image_faces))

        progress_bar = st.sidebar.progress(0)
        status_text = st.sidebar.empty()

        for i in range(1, 101):
            status_text.text("%i%% Complete" % i)
            progress_bar.progress(i)
            time.sleep(0.05)

        progress_bar.empty()


        # Get recognized face names
        face_names = {}
        recognized_faces = face_client.face.identify(image_face_ids, group_id)
        now = datetime.datetime.now()
        st.write('--------------Attendance--------------')
        st.write(f'Attendance date {now.strftime("%b %d %Y %H:%M:%S")}')
        for face in recognized_faces:
            if len(face.candidates) > 0:
                person_name = face_client.person_group_person.get(group_id, face.candidates[0].person_id).name
                face_names[face.face_id] = person_name
                st.write(person_name)

        # show recognized faces
        engine.show_recognized_faces(image_path, image_faces, face_names)
        if stop:
            break
        else:
            time.sleep(minutes*60)


def print_current_students(app_mode):
    st.header(app_mode)
    df = pd.read_csv('group_id.csv')
    groups = df['ID'].tolist()
    selection = st.selectbox('Select the Group list', groups)
    if selection == 'student':
        try:
            df = pd.read_csv('student_id.csv')
            st.dataframe(df)
        except:
            st.error('The group list is Empty')
    if selection == 'Teacher':
        try:
            df = pd.read_csv('teacher_id.csv')
            st.dataframe(df)
        except:
            st.error('The group list is Empty')

    if selection == 'Admin':
        try:
            df = pd.read_csv('teacher_id.csv')
            st.dataframe(df)
        except:
            st.error('The group list is Empty')

def main():
    # Create a face detection client.
    st.sidebar.title("What to do?")
    app_mode = st.sidebar.selectbox("Choose the app mode",
                                    ["Current Groups",
                                     "Create a New Group",
                                     "Add a New Member",
                                     "Student List",
                                     "Take Attendance from Picture"]
                                    )
    if app_mode == "Current Groups":
        print_current_groups(app_mode)
        st.sidebar.success('To continue select "Run the app".')
    elif app_mode == "Create a New Group":
        create_a_group_id(app_mode)
        st.sidebar.success('To continue select "Run the app".')
    elif app_mode == "Add a New Member":
        add_new_student(app_mode)
        st.sidebar.success('To continue select "Run the app".')
    elif app_mode == "Student List":
        print_current_students(app_mode)
        st.sidebar.success('To continue select "Run the app".')
    elif app_mode == "Take Attendance from Picture":
        detect_student_class(app_mode)
        st.sidebar.success('To continue select "Run the app".')


if __name__ == "__main__":

    from SessionState import get
    session_state = get(password='')
    st.title('Attendance List AutoApp')
    if session_state.password != 'pwd123':
        pwd_placeholder = st.sidebar.empty()
        pwd = pwd_placeholder.text_input("Password:", value="", type="password")
        session_state.password = pwd
        if session_state.password == 'pwd123':
            pwd_placeholder.empty()
            main()
        else:
            st.error("the password you entered is incorrect")
    else:
        cog_key = 'xxxxxxxxxxxxxxxxx'
        cog_endpoint = 'https://thesis-machine.cognitiveservices.azure.com/'
        face_client = FaceClient(cog_endpoint, CognitiveServicesCredentials(cog_key))
        main()


